from dataclasses import dataclass

from reactivex import combine_latest, operators as ops, Observable

from clochette import log
from clochette.application.store.CalendarConfigurationStore import CalendarConfigurationStore
from clochette.application.store.GeneralConfigurationStore import GeneralConfigurationStore
from clochette.domain.entity.delta.SnoozeDelta import SnoozeDelta
from clochette.domain.entity.configuration.CalendarConfiguration import CalendarConfiguration
from clochette.domain.entity.configuration.ThemeConfiguration import ThemeConfiguration
from clochette.framework.rx.Scheduler import scheduler
from clochette.infrastructure.mapper.Mapper import Mapper
from clochette.persist.configuration.ConfigurationWriterService import ConfigurationWriterService
from clochette.persist.configuration.dto.CalendarConfigurationDTO import CalendarConfigurationDTO
from clochette.persist.configuration.dto.GlobalConfigurationDTO import GlobalConfigurationDTO


@dataclass
class PersistConfigurationUseCase:
    """Use case to persist calendar configuration to disk."""

    _calendar_configuration_store: CalendarConfigurationStore
    _general_configuration_store: GeneralConfigurationStore
    _configuration_writer_service: ConfigurationWriterService
    _mapper: Mapper

    def persist_configuration(self) -> Observable[None]:
        """Retrieve current configuration from stores and persist to disk."""

        return combine_latest(
            self._calendar_configuration_store.calendars.pipe(ops.take(1)),
            self._general_configuration_store.snoozes.pipe(ops.take(1)),
            self._general_configuration_store.theme.pipe(ops.take(1)),
        ).pipe(
            ops.map(self._build_global_config_dto),
            ops.flat_map(self._configuration_writer_service.write),
            ops.do_action(on_error=lambda e: log.error(f"Failed to persist configuration", exc_info=e)),
            ops.subscribe_on(scheduler=scheduler),
        )

    def _build_global_config_dto(
        self, values: tuple[list[CalendarConfiguration], list[SnoozeDelta], ThemeConfiguration]
    ) -> GlobalConfigurationDTO:
        """Build GlobalConfigurationDTO from store values."""
        calendars, snoozes, theme = values
        calendar_dtos = [self._mapper.map(CalendarConfigurationDTO, cal) for cal in calendars]
        return GlobalConfigurationDTO(snoozes=snoozes, calendars=calendar_dtos, theme=theme)



from dataclasses import dataclass

from clochette.application.service.provider.dto.ProviderDTO import ProviderDTO
from clochette.provider.microsoft.configuration.MicrosoftSourceConfigurationService import (
    MicrosoftSourceConfigurationService,
)
from clochette.provider.microsoft.download.MicrosoftCalendarDownloaderService import (
    MicrosoftCalendarDownloaderService,
)
from clochette.provider.microsoft.dto.MicrosoftProviderType import MicrosoftProviderType
from clochette.provider.microsoft.ui.QMicrosoftOAuth2Component import (
    QMicrosoftOAuth2Component,
)


@dataclass
class MicrosoftProviderFactory:
    """Provider Factory for Microsoft calendars."""

    _download_calendar: MicrosoftCalendarDownloaderService
    _provider_configuration_mapper: MicrosoftSourceConfigurationService
    _authentication_component: QMicrosoftOAuth2Component

    def create(self) -> ProviderDTO:
        """Factory method to create a MicrosoftProviderDTO with all dependencies."""
        return ProviderDTO(
            download_calendar=self._download_calendar,
            provider_type=MicrosoftProviderType(),
            provider_configuration_mapper=self._provider_configuration_mapper,
            display_name="Microsoft",
            authentication_component=self._authentication_component,
        )



from dataclasses import dataclass

from reactivex import operators as ops, Observable

from clochette import log
from clochette.application.repository.CalendarRepository import CalendarRepository
from clochette.application.store.CalendarConfigurationStore import CalendarConfigurationStore
from clochette.application.usecase.PersistConfigurationUseCase import PersistConfigurationUseCase
from clochette.application.usecase.ScheduleCalendarDownloadUseCase import ScheduleCalendarDownloadUseCase
from clochette.domain.entity.configuration.CalendarConfiguration import CalendarConfiguration
from clochette.framework.rx.Scheduler import scheduler


@dataclass
class AddCalendarUseCase:
    """Use case to add a new calendar to the configuration store."""

    _calendar_configuration_store: CalendarConfigurationStore
    _persist_configuration_usecase: PersistConfigurationUseCase
    _schedule_calendar_download_usecase: ScheduleCalendarDownloadUseCase
    _calendar_repository: CalendarRepository

    def add_calendar(self, calendar: CalendarConfiguration) -> Observable[None]:
        """Add a new calendar to the configuration store and persist to disk."""

        log.info(f"Adding new calendar: {calendar}")

        def update_store(calendars: list[CalendarConfiguration]) -> Observable[None]:
            # Add new calendar to the list
            updated_calendars = calendars + [calendar]
            self._calendar_configuration_store.set_calendars(updated_calendars)
            self._calendar_configuration_store.add_calendar(calendar)

            # Persist to disk
            return self._persist_configuration_usecase.persist_configuration().pipe(
                ops.do_action(on_next=lambda _: self._schedule_calendar_download_usecase.schedule_calendar(calendar)),
            )

        return self._calendar_configuration_store.calendars.pipe(
            ops.take(1),
            ops.do_action(
                on_next=lambda _: self._calendar_repository.add_calendar(calendar),
            ),
            ops.flat_map(lambda x: update_store(x)),
            ops.do_action(
                on_error=lambda e: log.error(f"Error while adding calendar named: {calendar.name}", exc_info=e)
            ),
            ops.subscribe_on(scheduler=scheduler),
        )
